@@ -17,16 +17,31 @@
 #'    IMPORTANT!!!!! ->
 #'        POST is the only option for this parameter if you want to
 #'        use source or code parameters.
-#' @param sleep Numer of seconds by which to pause between calls. Defaults to 0
+#' @param sleep Number of seconds by which to pause between calls. Defaults to 0
 #' 		seconds. Use when doing many calls in a for loop ar lapply type call.
 #' @param splitby Number by which to split species list for querying the TNRS.
 #' @param verbose Verbosity or not (default \code{TRUE})
 #' @param ... Curl options to pass in \code{\link[httr]{GET}} or
 #' \code{\link[httr]{POST}}
 #'
-#' @return data.frame of results from TNRS plus the name submitted.
+#' @return data.frame of results from TNRS plus the name submitted, with
+#' rows in order of user supplied names, though those with no matches are
+#' dropped
 #' @details If there is no match in the Taxosaurus database, nothing is
-#'    returned, so you will not get anything back for non-matches.
+#' returned, so you will not get anything back for non-matches.
+#'
+#' TNRS doesn't provide any advice about the occurrence of homonyms when
+#' queries have no indication of a taxonomic name's authority. So if there
+#' is any chance of a homonym, you probably want to send the authority as
+#' well, or use \code{\link{gnr_resolve}}. For example,
+#' \code{tnrs(query="Jussiaea linearis", source="iPlant_TNRS")} gives result of
+#' \emph{Jussiaea linearis (Willd.) Oliv. ex Kuntze}, but there is a
+#' homonym. If you do
+#' \code{tnrs(query="Jussiaea linearis Hochst.", source="iPlant_TNRS")} you
+#' get a direct match for that name. So, beware that there's no indication
+#' of homonyms.
+#' @references \url{http://taxosaurus.org/}
+#' @seealso \code{\link{gnr_resolve}}
 #' @examples \dontrun{
 #' mynames <- c("Helianthus annuus", "Poa annua", "Mimulus bicolor")
 #' tnrs(query = mynames, source = "iPlant_TNRS")
@@ -67,7 +82,7 @@ tnrs <- function(query = NA, source = NULL, code = NULL, getpost = "POST",
     if (getpost == "get") {
       if (!any(is.na(x))) {
         query2 <- paste(str_replace_all(x, ' ', '+'), collapse = '%0A')
-        args <- tc(list(query = query2))
+        args <- tc(list(query = query2, source = source, code = code))
         out <- GET(url, query = args, ...)
         error_handle(out)
         retrieve <- out$url
@@ -77,8 +92,9 @@ tnrs <- function(query = NA, source = NULL, code = NULL, getpost = "POST",
     } else {
       loc <- tempfile(fileext = ".txt")
       write.table(data.frame(x), file = loc, col.names = FALSE, row.names = FALSE)
-      args <- tc(list(file = upload_file(loc), source = source, code = code))
-      out <- POST(url, body = args, config(followlocation = 0), ...)
+      args <- tc(list(source = source, code = code))
+      body <- tc(list(file = upload_file(loc)))
+      out <- POST(url, query = args, body = body, config(followlocation = 0), ...)
       error_handle(out)
       tt <- con_utf8(out)
       message <- jsonlite::fromJSON(tt, FALSE)[["message"]]
@@ -117,16 +133,19 @@ tnrs <- function(query = NA, source = NULL, code = NULL, getpost = "POST",
 
   if (length(query) < 1 || is.na(query)) stop("Please supply at least one name", call. = FALSE)
 
-  if (getpost == "get" && length(query) > 75 | length(query) > 30 && getpost == "post") {
+  if (getpost == "get" && length(query) > 75 |
+      length(query) > 30 && getpost == "post") {
     species_split <- slice(query, by = splitby)
 
     out <- lapply(species_split, function(x) mainfunc(x, ...))
     tmp <- data.frame(rbindlist(out), stringsAsFactors = FALSE)
-    setNames(tmp, tolower(names(tmp)))
   } else {
     tmp <- mainfunc(query, ...)
-    setNames(tmp, tolower(names(tmp)))
   }
+  tmp <- tmp[match(query, tmp$submittedName), ]
+  tmp <- na.omit(tmp)
+  row.names(tmp) <- NULL
+  stats::setNames(tmp, tolower(names(tmp)))
 }
 
 # Function to parse results
